#!/usr/bin/python

from OpenGL import GL
import random
from shape import * 
from attributes import *
from basic_shapes import *

# Three important elements: house, wall, terrace
# elements are modeled separately (top-down) and then placed together (half-way
# bottom up)

# house -> facades (input params: width, height, depth)
# facades -> facade (front, back, side, side)
# front facade -> doors and windows (number of doors and windows are params)
#               or just one pillar
# back facade -> windows 
# side facase -> rectangular or triangle
# (each house has an attribute orientation)
class House(Shape):
    def __init__(self, attr=None, id=-1):
        Shape.__init__(self)
        self.id = id
        self.attr = attr

        self.trans = [0, 0, 0]
        self.rot = 0

        self.facades = None
        self.roof = None
 
    def set_attr(self, attr):
        self.attr = attr

    def set_id(self, id):
        self.id = id

    def init_attributes(self):
        self.attr = HouseAttributes()
        
    def generate(self):
        if self.attr == None:
            self.init_attributes()
        if self.facades == None:
            self.facades = Facades(self.attr)
        if self.roof == None:
            self.roof = Roof(self.attr)

        self.facades.generate()
        self.roof.generate(self.attr.side_facade_type)

    
    def set_translation(self, trans):
        self.trans = trans

    def set_rotation(self, rot):
        self.rot = rot

    def render(self):
        if self.facades:
            GL.glTranslate(self.trans[0], self.trans[1], self.trans[2])
            GL.glRotate(self.rot, 0, 1, 0)
            GL.glPushMatrix()
            self.facades.render()
            GL.glPopMatrix()

        if self.roof:
            GL.glPushMatrix()
            GL.glTranslate(0.0, self.attr.height, 0.0)
            self.roof.render()
            GL.glPopMatrix()


class Facades(Shape):
    def __init__(self, attr):
        Shape.__init__(self)
        self.attr = attr

    def generate(self):
        self.width = self.attr.width
        self.height = self.attr.height
        self.depth = self.attr.depth
        self.roof_height = self.attr.roof_height

        if self.attr.num_front_doors == 0:
            self.attr.front_facade_type = PILLAR_FRONT
        else:
            self.attr.front_facade_type = REGULAR_FRONT

        self.front = FrontFacade(self.width, self.height,
                                    self.attr.front_facade_type)
        self.front.set_num_doors(self.attr.num_front_doors)
        self.front.set_door_width(self.attr.door_width)
        self.front.set_door_height(self.attr.door_height)
        self.front.set_num_windows(self.attr.num_front_wins)
        self.front.set_win_width(self.attr.win_width)
        self.front.set_win_frame_width(self.attr.win_frame_width)
        self.front.set_win_frame_height(self.attr.win_frame_height)
        self.front.set_win_frame_base(self.attr.win_frame_base)

        self.back = Facade(self.width, self.height)
        self.back.set_num_doors(self.attr.num_back_doors)
        self.back.set_door_width(self.attr.door_width)
        self.back.set_door_height(self.attr.door_height)
        self.back.set_num_windows(self.attr.num_back_wins)
        self.back.set_win_width(self.attr.win_width)
        self.back.set_win_frame_width(self.attr.win_frame_width)
        self.back.set_win_frame_height(self.attr.win_frame_height)
        self.back.set_win_frame_base(self.attr.win_frame_base)

        if self.attr.num_side_wins > 1:
            self.attr.side_facade_type = RECTANGULAR_SIDE
        else:
            if self.attr.side_facade_type == UNDEFINED_SIDE:
                if (random.random() < 0.2):
                    self.attr.side_facade_type = RECTANGULAR_SIDE
                else:
                    self.attr.side_facade_type = SHARP_SIDE

        self.left = SideFacade(self.depth, self.height, \
                                self.roof_height, self.attr.side_facade_type)
        self.left.set_num_windows(self.attr.num_side_wins)
        self.left.set_win_width(self.attr.win_width)
        self.left.set_win_frame_width(self.attr.win_frame_width)
        self.left.set_win_frame_height(self.attr.win_frame_height)
        self.left.set_win_frame_base(self.attr.win_frame_base)

        self.right = SideFacade(self.depth, self.height, \
                                self.roof_height, self.attr.side_facade_type)
        self.right.set_num_windows(self.attr.num_side_wins)
        self.right.set_win_width(self.attr.win_width)
        self.right.set_win_frame_width(self.attr.win_frame_width)
        self.right.set_win_frame_height(self.attr.win_frame_height)
        self.right.set_win_frame_base(self.attr.win_frame_base)

        self.front.generate()
        self.back.generate()
        self.left.generate()
        self.right.generate()


    def render(self):
        GL.glMatrixMode(GL.GL_MODELVIEW)

        GL.glPushMatrix()
        GL.glTranslate(0.0, 0.0, 0.5 * self.depth)
        self.front.render()
        GL.glPopMatrix()

        GL.glPushMatrix()
        GL.glTranslate(0.0, 0.0, -0.5 * self.depth)
        GL.glRotate(180, 0, 1, 0)
        self.back.render()
        GL.glPopMatrix()

        GL.glPushMatrix()
        GL.glTranslate(-0.5 * self.width, 0.0, 0.0)
        GL.glRotate(90, 0, 1, 0)
        self.left.render()
        GL.glPopMatrix()

        GL.glPushMatrix()
        GL.glTranslate(0.5 * self.width, 0.0, 0.0)
        GL.glRotate(-90, 0, 1, 0)
        self.right.render()
        GL.glPopMatrix()

class Facade(Shape):
    def __init__(self, width, height):
        Shape.__init__(self)
        self.width = width
        self.height = height
        self.num_windows = 0
        self.win_width = 0
        self.win_frame_width = 0
        self.win_frame_height = 0
        self.win_frame_base = 0
        self.num_doors = 0
        self.door_width = 0
        self.door_height = 0

    def set_num_windows(self, num_windows):
        self.num_windows = num_windows

    def set_win_width(self, win_width):
        self.win_width = win_width

    def set_win_frame_width(self, win_frame_width):
        self.win_frame_width = win_frame_width

    def set_win_frame_height(self, win_frame_height):
        self.win_frame_height = win_frame_height

    def set_win_frame_base(self, win_frame_base):
        self.win_frame_base = win_frame_base

    def set_num_doors(self, num_doors):
        self.num_doors = num_doors

    def set_door_width(self, door_width):
        self.door_width = door_width

    def set_door_height(self, door_height):
        self.door_height = door_height

    def generate(self):
        self.clear()

        # (1r, blah, 1r){wall | ... | wall}
        if self.num_doors == 0:
            if self.num_windows == 0:
                # blah -> walls
                wall = Wall(self.width, self.height)
                self.add_shape(wall)

            else:    
                # blah -> window(s) 
                # blah -> wall windows(s) wall
                total_window_width = self.win_width * self.num_windows
                if total_window_width > self.width:
                    self.win_width = (self.width - self.num_windows - 1) / self.num_windows
                    self.win_frame_width = 0.5 * self.win_width
                    self.win_frame_height = 0.5 * self.win_width
                    total_window_width = self.win_width * self.num_windows

                num_walls = self.num_windows + 1
                total_wall_width = self.width - total_window_width
                wall_width = total_wall_width / num_walls

                wall = Wall(wall_width, self.height)
                win = Window(self.win_frame_width, self.win_frame_height, \
                            self.win_frame_base, self.win_width, self.height)

                self.add_shape(wall, [-0.5 * (self.width - wall_width), 0, 0])
                
                trans = 0.5 * (wall_width + self.win_width)
                for i in xrange(self.num_windows):
                    self.add_shape(win, [trans, 0, 0])
                    self.add_shape(wall, [trans, 0, 0])

        else:
            if self.num_windows == 0:
                # blah -> door(s) separated by walls
                total_door_width = self.door_width * self.num_doors
                if total_door_width > self.width:
                    self.door_width = (self.width - self.num_doors - 1) / self.num_doors
                    total_door_width = self.door_width * self.num_doors

                num_walls = self.num_doors + 1
                total_wall_width = self.width - total_door_width
                wall_width = total_wall_width / num_walls

                wall = Wall(wall_width, self.height)
                door = Door(self.door_width, self.door_height, self.height)

                self.add_shape(wall, [-0.5 * (self.width - wall_width), 0, 0])

                trans = 0.5 * (wall_width + self.door_width)
                for i in xrange(self.num_doors):
                    self.add_shape(door, [trans, 0, 0])
                    self.add_shape(wall, [trans, 0, 0])

            else:
                # blah -> (1r, repeat(d_width, xr), d_width, 1r){windows, ..} 
                num_win_slots = self.num_doors + 1
                rem = self.num_windows % num_win_slots
                extra = 0
                if rem <> 0:
                    extra = num_win_slots - rem
                self.num_windows += extra
                num_win_per_slot = self.num_windows / num_win_slots
                wall_width = (self.width - self.num_doors * self.door_width \
                                - self.num_windows * self.win_width) / 2
                if wall_width < 1:
                    wall_width = 1
                    self.win_width = (self.width - self.num_doors * self.door_width \
                                        - 2) / self.num_windows
                                
                wall = Wall(wall_width, self.height)
                door = Door(self.door_width, self.door_height, self.height)
                win = Window(self.win_frame_width, self.win_frame_height, \
                             self.win_frame_base, self.win_width, self.height)

                self.add_shape(wall, [-0.5 * (self.width - wall_width), 0, 0])
                self.add_shape(win, [0.5 * (wall_width + self.win_width), 0, 0])

                for j in xrange(num_win_per_slot-1):
                    self.add_shape(win, [self.win_width, 0, 0])

                for i in xrange(self.num_doors):
                    self.add_shape(door, [0.5 * (self.win_width + self.door_width), 0, 0])
                    self.add_shape(win, [0.5 * (self.door_width + self.win_width), 0, 0])

                    for j in xrange(num_win_per_slot-1):
                        self.add_shape(win, [self.win_width, 0, 0])

                self.add_shape(wall, [0.5 * (wall_width + self.win_width), 0, 0])


class FrontFacade(Facade):
    def __init__(self, width, height, type):
        Facade.__init__(self, width, height)
        self.type = type

    def generate(self):
        self.clear()

        if self.type == REGULAR_FRONT:
            Facade.generate(self)

        elif self.type == PILLAR_FRONT:
            pillar = Pillar(4, self.height)
            self.add_shape(pillar)


class SideFacade(Facade):
    def __init__(self, width, height, roof_height, type):
        Facade.__init__(self, width, height)
        self.roof_height = roof_height
        self.type = type

    def generate(self):
        self.clear()

        if self.type == RECTANGULAR_SIDE:
            Facade.generate(self)

        elif self.type == SHARP_SIDE:
            wall = SideWall(self.width, self.height, self.roof_height, \
                    self.win_frame_width, self.win_frame_height)
            self.add_shape(wall)

class Roof(Shape):
    def __init__(self, attr):
        Shape.__init__(self)
        self.attr = attr

    def generate(self, side_type):
        self.clear()

        self.width = self.attr.width
        self.height = self.attr.roof_height
        self.depth = self.attr.depth
        if side_type == RECTANGULAR_SIDE:
            roof = RecRoof(self.width, self.height, self.depth)
        else: 
            roof = SharpRoof(self.width, self.height, self.depth)

        self.add_shape(roof)
